import os
from flask import Flask, render_template, request, jsonify, url_for
from sendgrid import SendGridAPIClient
from database import db
from models import Subscriber

app = Flask(__name__)

# Configure database
app.secret_key = os.environ.get("FLASK_SECRET_KEY") or "a secret key"
app.config["SQLALCHEMY_DATABASE_URI"] = os.environ.get("DATABASE_URL")
app.config["SQLALCHEMY_ENGINE_OPTIONS"] = {
    "pool_recycle": 300,
    "pool_pre_ping": True,
}

db.init_app(app)

@app.route('/')
def index():
    return render_template('index.html')

def send_confirmation_email(subscriber):
    print(f"\n--- Starting email confirmation process for {subscriber.email} ---")
    
    token = subscriber.generate_confirmation_token()
    confirm_url = url_for('confirm_email', token=token, _external=True)
    print(f"Generated confirmation URL: {confirm_url}")
    
    sg = SendGridAPIClient(os.environ.get('SENDGRID_API_KEY'))
    
    message = {
        'personalizations': [{
            'to': [{'email': subscriber.email}],
            'subject': 'Confirm your CHIPS subscription'
        }],
        'from': {'email': 'support@chipsmobile.com', 'name': 'CHIPS Support'},
        'content': [{
            'type': 'text/plain',
            'value': f'''Please confirm your email address for CHIPS updates.

To confirm your subscription, please visit:
{confirm_url}

If you didn't request this, you can safely ignore this email.

To unsubscribe, reply with "unsubscribe" in the subject line.

CHIPS
123 Innovation Way
San Francisco, CA 94105
United States'''
        }, {
            'type': 'text/html',
            'value': render_template('email/confirm.html', confirm_url=confirm_url)
        }],
        'headers': {
            'List-Unsubscribe': f'<mailto:support@chipsmobile.com?subject=unsubscribe&body={subscriber.email}>',
            'Precedence': 'bulk',
            'X-Auto-Response-Suppress': 'OOF, DR, RN, NRN, AutoReply'
        }
    }
    
    try:
        response = sg.client.mail.send.post(request_body=message)
        print(f"Email sent. Status code: {response.status_code}")
        return True
    except Exception as e:
        print(f"Error sending email: {str(e)}")
        raise e

@app.route('/subscribe', methods=['POST'])
def subscribe():
    try:
        email = request.form.get('email')
        if not email:
            return jsonify({'error': 'Email is required'}), 400
        
        existing_subscriber = Subscriber.query.filter_by(email=email).first()
        if existing_subscriber:
            if existing_subscriber.is_confirmed:
                return jsonify({'error': 'Email already subscribed'}), 400
            send_confirmation_email(existing_subscriber)
            return jsonify({'message': 'Confirmation email resent!'}), 200
        
        new_subscriber = Subscriber(email=email)
        db.session.add(new_subscriber)
        db.session.commit()
        
        send_confirmation_email(new_subscriber)
        
        return jsonify({'message': 'Please check your email to confirm your subscription!'}), 200
    except Exception as e:
        return jsonify({'error': str(e)}), 500

@app.route('/confirm/<token>')
def confirm_email(token):
    try:
        subscriber = Subscriber.query.filter_by(confirmation_token=token).first()
        if not subscriber:
            return render_template('error.html', message='Invalid confirmation link'), 400
        
        subscriber.is_confirmed = True
        subscriber.confirmation_token = None
        db.session.commit()
        
        return render_template('confirmed.html')
    except Exception as e:
        return render_template('error.html', message=str(e)), 500

# Initialize database
with app.app_context():
    db.create_all()

if __name__ == '__main__':
    app.run(host='0.0.0.0', port=5000)
